*******************************************************
*  													  *
*  How to undertake an external validation study to   *
*  evaluate the performance of a clinical prediction  *
*  model - 2022										  *
*  													  *
*  Binary outcome example - Stata 16				  *
*  													  *
*  RDR, LA, KIES, JE, PD, GPM, LJB, GSC				  *
*  													  *
*******************************************************

*set working directory
cd " " 

*read in FH Gusto data, read from hbiostat.org/data
import delim using "FHgusto.csv", clear

*label regions
label define regions ///
		1	"US, West" ///
		2	"Australia & New Zealand" ///
		3	"Belgium" ///
		4	"Canada" ///
		5	"France, Spain & Switzerland" ///
		6	"Germany & Poland" ///
		7	"US, Great Lakes" ///
		8	"Israel" ///
		9	"US, Mid-Atlantic" ///
		10	"US, Middle South" ///
		11	"US, Midwest" ///
		12	"US, Northeast" ///
		13	"Netherlands" ///
		14	"US, Southeast" ///
		15	"US, Southwest" ///
		16	"UK & Ireland"

label values regl regions		

*recode hypertension variable from 1/2 to 0/1
replace htn = htn-1
*recode previous mi - currently string
gen PMI = 0 
	replace PMI = 1 if pmi == "yes"

*drop model development data: Western US
drop if regl == 1
		
*generate variable for LP (linear predictor) of prediction model
gen LP = -4.365084 ///
		- (0.3280145*[htn==1]) ///
		+ (0.5443634*[sex=="female"]) ///
		+ (0.007297*(((age/10)^3)-221.10694)) ///
		+ (0.692547*[killip=="II"])+(2.746477*[killip=="III"])+(2.808144*[killip=="IV"]) ///
		+ (1.386137*[hyp==1]) ///
		+ (0.6522851*[hrt==1]) ///
		+ (0.6847723*[PMI==1]) ///
		+ (0.215318*(ste-3.99954))

*transform to probability scale
gen prob = exp(LP)/(1+exp(LP))

*******************************************************

*~._.~*~._.~*~._.~*

*plot total LP distribution with summary statistics

sum LP, detail
	local mean = r(mean)
	local median = r(p50)
	local sd = r(sd)
	local lq = r(p25)
	local uq = r(p75)

twoway (hist LP, dens width(0.3) color(navy%60)) ///
		(kdensity  LP , lcolor(navy) bwidth(0.5)), ///
		xtitle("Linear predictor value") scheme(s1color) ///
		text(0.3 1.5 ///
		"Mean = `: display %3.2f `mean''" ///
		"Median = `: display %3.2f `median''" ///
		"SD = `: display %3.2f `sd''" ///
		"IQR = `: display %3.2f `lq'' to `: display %3.2f `uq''" ///
		, place(se) just(left))	legend(off)	

graph export "binary_LPDist_all.png", as(png) replace		

*~._.~*~._.~*~._.~*


*~._.~*~._.~*~._.~*

*plot LP distribution by outcome, with summary statistics

sum LP if day30 == 0, detail
	local mean_0 = r(mean)
	local median_0 = r(p50)
	local sd_0 = r(sd)
	local lq_0 = r(p25)
	local uq_0 = r(p75)
	
sum LP if day30 == 1, detail
	local mean_1 = r(mean)
	local median_1 = r(p50)
	local sd_1 = r(sd)
	local lq_1 = r(p25)
	local uq_1 = r(p75)

twoway  (hist LP if day30 == 0, dens color(navy%40) lcolor(black) width(0.3)) ///
		(hist LP if day30 == 1, dens color(dkgreen%40) lcolor(black) width(0.3)) ///
		(kdensity  LP if day30 == 0, lcolor(navy) bwidth(0.5)) ///
		(kdensity  LP if day30 == 1, lcolor(dkgreen) bwidth(0.5)), ///
		xtitle("Linear predictor value") scheme(s1color) ///
		legend(order(1 "No death within 30 days" 2 "Death within 30 days" )) ///
		text(0.3 1.5 ///
		"No death:" ///
		"  Mean = `: display %3.2f `mean_0''" ///
		"  Median = `: display %3.2f `median_0''" ///
		"  SD = `: display %3.2f `sd_0''" ///
		"  IQR = `: display %3.2f `lq_0'' to `: display %3.2f `uq_0''" ///
		" " ///
		"Death:" ///
		"  Mean = `: display %3.2f `mean_1''" ///
		"  Median = `: display %3.2f `median_1''" ///
		"  SD = `: display %3.2f `sd_1''" ///
		"  IQR = `: display %3.2f `lq_1'' to `: display %3.2f `uq_1''" ///		
		, place(se) just(left))		

graph export "binary_LPDist_byoutcome.png", as(png) replace		

*~._.~*~._.~*~._.~*

*******************************************************	
	
*calibration plot across all regions
pmcalplot prob day30 , ci bin(10) name(calibration_plot, replace) nostat  ///
					xtitle("Estimated probability", size(medsmall)) ///
					yscale(range(-.2(.2)1)) ylabel(0(.2)1) ytitle("Observed probability", size(medsmall)) ///
					legend(order(2 3 4) lab(2 "Groups") lab(3 "Calibration curve (Lowess)") lab(4 "95% confidence interval")) 

graph export "binary_calplot.png", as(png) replace		
	
*generate calibration plot with histogram

gen event = day30

gen histeventn = . 
gen histnoneventn = .
gen histbinvar = 1

local bin = 50
local diff = 1/`bin'
local grp = 1
forvalues i = 0(`diff')1 {
	local difflow = `i'
	local diffhigh = `i'+`diff'
	replace histbinvar = `grp' if prob>=`difflow' & prob<`diffhigh'
	local grp = `grp'+1
}

forvalues i = 1/`bin' {
	qui count if event ==1 & histbinvar==`i' 
	qui replace histeventn=r(N) if histbinvar==`i' 
	qui count if event ==0 & histbinvar==`i' 
	qui replace histnoneventn=r(N) if histbinvar==`i'
	}
	
*scale to fit along the bottom of the calibration plot	
qui count if event == 1 
	local totale = r(N)
qui count if event == 0 
	local totalne = r(N)

qui replace histeventn = (histeventn/`totale')
qui replace histnoneventn = (histnoneventn/`totalne')

gen histexp = `diff'*histbinvar
gen droplinebase = -0.1

*scale to fit along the bottom of the calibration plot	
qui sum histeventn
	local maxe = r(max)
qui sum histnoneventn
	local maxne = r(max)
	local maxout = max(`maxe', `maxne')

qui replace histnoneventn = -1*histnoneventn

qui replace histeventn = droplinebase + (histeventn/`maxout')*(0.1)
qui replace histnoneventn = droplinebase +(histnoneventn/`maxout')*(0.1)	
	
****
egen groups2 = cut(LP), group(10)

bysort groups2: egen ave_exp = mean(prob)	
bysort groups2: egen ave_obs = mean(event)	
bysort groups2: egen n_obs = count(event)	

gen obs_se = ((ave_obs*(1-ave_obs))/n_obs)^0.5
gen lb_ave_obs = max(0,(ave_obs - (1.96*obs_se)))
gen ub_ave_obs = min(1,(ave_obs + (1.96*obs_se)))
	
qui running day30 prob , span(1) ci generate(smooth) gense(smooth_se) replace nog
	gen smooth_lci = smooth - (1.96*smooth_se)
	replace smooth_lci=0 if smooth_lci<0
	gen smooth_uci = smooth + (1.96*smooth_se)
	replace smooth_uci=1 if smooth_uci>1
	
twoway (line smooth prob if smooth <1, sort lcol(midblue)) ///
	(rarea smooth_lci smooth_uci prob, sort fc(midblue%20) lc(midblue%20)) ///
	(scatter ave_obs ave_exp, msymbol(Oh) mcol(dkgreen)) ///
	(rspike ub_ave_obs lb_ave_obs ave_exp, lcol(dkgreen)) ///
	(rbar histeventn droplinebase histexp if (histexp< 1) & (histexp> 0), barw(0.02) msym(i) yline(-0.1, lw(vthin)) text(-.05 1 "Death within 30 days", place(w) size(vsmall)) lw(none) col(maroon)) ///
	(rbar histnoneventn droplinebase histexp if (histexp< 1) & (histexp> 0), barw(0.02) msym(i) text(-0.15 1 "No death within 30 days", place(w) size(vsmall)) lw(none) col(forest_green)) ///
	(function y = x, ra(0 1) clpat(-) clcol(gs5)) ///
	, ylabel(0(0.2)1, labsize(small)) xlabel(0(0.2)1, labsize(small)) aspect(1.2) ///
	yscale(range(-0.2 1)) xscale(range(0 1)) ///
	ytitle("Observed risk", size(vsmall)) xtitle("Estimated risk", size(vsmall)) title("") ///
	graphregion(color(white))  ///
	legend(on pos(3) col(1) order(3 "Groups"  4 "95% confidence interval - group" 1 "Calibration curve" 2 "95% confidence interval - curve" 7 "Ideal calibration") size(vsmall))
	

graph export "binary_calplot.png", as(png) replace			

*******************************************************	

*decision curve analysis
dca day30 prob,  prob(yes) smooth scheme(s1color) ///
	lcolor(black gs8 dkgreen) lpattern(dash solid solid) ///
	legend(col(1) order(1 "Treat all" 2 "Treat none" 3 "Treat per model")) ///
	saving("binary_dca", replace) ymin(-0.015)
	
*use "binary_dca", clear

graph export "binary_dca.png", as(png) replace		
	
*******************************************************		

*predictive performance statistics 


*R2 Cox-Snell/R2 Nagelkerke
capture program drop myboot
program define myboot, rclass
 preserve 
		logit day30 LP, nocons iterate(0) from(LP=1)
		local ll_full = e(ll)
		sum day30
		local E = r(mean)*r(N)
		local n = r(N)
		local ll_null = `E'*ln(`E'/`n') + (`n'-`E')*ln(1-(`E'/`n'))
		local LR = -2*(`ll_null'-`ll_full')
		local r2cs = 1 - exp(-`LR'/`n')
		local max_r2cs = 1 - exp((2*`ll_null')/`n')
		local r2N = `r2cs'/`max_r2cs'
  return scalar r2cs = `r2cs'
  return scalar r2N = `r2N'
 restore
end

*bootstrap confidence intervals for r2 statistics
bootstrap r(r2cs) r(r2N), reps(1000) seed(45672): myboot

*~._.~*~._.~*~._.~*

*Brier score
brier day30 prob
bootstrap r(brier) , reps(1000) seed(42513): brier day30 prob

*~._.~*~._.~*~._.~*

*Calibration-in-the-large
logistic day30, offset(LP) coef

*~._.~*~._.~*~._.~*

*Observed/Expected (O/E)
summarize day30
	local obs = r(mean)
summarize prob
	local exp = r(mean)
	local N = r(N)
di "O/E = " `obs'/`exp'

local OE =  `obs'/`exp'
local se_ln_oe = sqrt((1-`obs')/(`N')*`obs')

di "95% ci = " exp(ln(`OE') - 1.96*`se_ln_oe') " to "exp(ln(`OE') + 1.96*`se_ln_oe')

*~._.~*~._.~*~._.~*

*Calibration slope
logit day30 LP

*~._.~*~._.~*~._.~*

*Integrated Calibration Index (ICI)
lowess day30 prob, generate(p_cal) 
gen ici = abs(p_cal - prob)
sum ici
di "95% ci" r(mean) - 1.96*(r(sd)/sqrt(r(N))) " to " r(mean) + 1.96*(r(sd)/sqrt(r(N)))

*~._.~*~._.~*~._.~*

*C-statistic (AUROC)
roctab day30 prob


******************************************************************************